/****************************************************************************
 ** Filter - class
 **
 **   Created : Sat Jan 10 2009
 **        by : Varol Okan using kate editor
 ** Copyright : (c) Varol Okan
 ** License   : GPL v 2.0
 **
 ** Class to implement the base for additional filter.
 ** This class also includes the basic CrossFade, and
 ** KenBurns filter.
 **
 ** Derived class AlphaFilter will handle gray scale
 ** filter.
 **
 ****************************************************************************/

#ifndef RENDER_FILTER_H
#define RENDER_FILTER_H

// These are the currently supported filter types
#define FILTER_CROSS_FADE "crass_fade"
#define FILTER_KEN_BURNS  "ken_burns"
#define FILTER_ALPHA_FADE "alpha_fade"

#include "../qdvdauthor/xml_slideshow.h"

// Forward declaration of classes.
class QImage;

namespace Encoder { class FFmpeg; }

namespace Render
{

class Filter
{
public:
  // The audio transition while the filter is being displayed
  enum enAudioTrans { VolumeFull, VolumeNull, VolumeUp, VolumeDown };
             Filter ( );
  virtual   ~Filter ( );
  static Filter *create ( CXmlSlideshow *, CXmlSlideshow::filter_struct * );

  virtual QStringList getFilterNames ( );
  virtual bool exec ( Encoder::FFmpeg *, QImage &, QImage &, int, enAudioTrans, QImage *p=NULL );
};

class CrossFadeFilter : public Filter
{
public:
           CrossFadeFilter ( );
  virtual ~CrossFadeFilter ( );

  virtual QStringList getFilterNames ( );
  virtual bool exec ( Encoder::FFmpeg *, QImage &, QImage &, int, enAudioTrans, QImage *p=NULL );
};

class KenBurnsFilter : public Filter
{
public:
           KenBurnsFilter ( );
  virtual ~KenBurnsFilter ( );

  virtual QStringList getFilterNames ( );
  virtual bool exec ( Encoder::FFmpeg *, QImage &, QImage &, int, enAudioTrans, QImage *p=NULL );
};

class AlphaFilter : public Filter
{
public:
           AlphaFilter ( );
           AlphaFilter ( QString );
  virtual ~AlphaFilter ( );

  virtual QStringList getFilterNames ( );
  virtual bool exec ( Encoder::FFmpeg *, QImage &, QImage &, int, enAudioTrans, QImage *p=NULL );

private:
  // This one will simply transition rom one image to the next
  void augmentAlpha ( int, QImage &, QImage &, QImage &, QImage & );
  // This one will pass through the background
  void augmentAlpha ( int, int, QImage &, QImage &, QImage &, QImage &, QImage * );

private:
  QString m_qsFilterName;
};

}; // end of namespace Encoder

#endif // RENDER_FILTER_H
